/*
 * Access to memory cards.
 *
 * Copyright (C) 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <mutex>

#include <l4/devices/boot.h>
#include <l4/devices/dma.h>
#include <l4/devices/msc-generic.h>
#include <systypes/base.h>



#ifdef __cplusplus

class MscRegionOperations
{
protected:
  std::mutex _lock;
  Msc_channel *_msc_channel;
  uint8_t _card;
  struct dma_region _region;

public:
  explicit MscRegionOperations(Msc_channel *msc_channel, uint8_t card,
                               struct dma_region region);

  long get_partition(uint8_t partition, uint32_t *start_block,
                     uint32_t *num_blocks);

  long read_partition_table(struct partition_table_entry **entry);

  void read(l4_addr_t vaddr, l4re_dma_space_dma_addr_t paddr, offset_t pos,
            offset_t size);

  void write(l4_addr_t vaddr, l4re_dma_space_dma_addr_t paddr, offset_t pos,
             offset_t size);
};

#endif /* __cplusplus */
